#!/usr/bin/env python3
# Written by Pier Angelo Vendrame <vogliadifarniente@gmail.com> in
# February 2023.
# Any copyright is dedicated to the Public Domain.
# https://creativecommons.org/publicdomain/zero/1.0/
#
# Needs python-telegram-bot[job-queue].

from http.server import HTTPServer, BaseHTTPRequestHandler
from threading import Thread

from telegram import Update
from telegram.ext import (
    ApplicationBuilder,
    CallbackContext,
    JobQueue,
    CommandHandler,
    ContextTypes,
)

# Add here the token of your bot
TOKEN = "REDACTED"
# Change it to your Telegram ID. It will be used to send messages to.
# You can use the `/myid` command of the bot to know your ID.
# Probably there is a way to use "@username", instead, but I wanted to
# implement the bot quickly, and echoing the ID was faster.
CHAT_ID = 0


# Handle the `/myid` command.
# It can be used to get the user ID to fill the CHAT_ID variable.
async def myid(update: Update, context: ContextTypes.DEFAULT_TYPE):
    await context.bot.send_message(
        chat_id=update.effective_chat.id,
        text=f"Your chat id is {update.effective_chat.id}",
    )


# The job handler that sends the message when some motion is detected.
async def job_handler(context: CallbackContext):
    # Change your message here
    await context.bot.send_message(
        chat_id=CHAT_ID, text="È stato rilevato del movimento!!!"
    )


# The queue used to schedule message from the HTTP server
queue = JobQueue()


# The handler for HTTP requests
class MotionHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        # If we just want to send a message when some motion is
        # detected, we could simply ignore the path of the request.
        # However, in this way, we can reuse the bot also for other
        # purposes, if needed.
        if self.path == "/motion_started":
            self.send_response(200)
            self.end_headers()
            self.wfile.write(b"Message queued")
            queue.run_once(job_handler, 0)
        else:
            # Do not show any local file! Just send a 404 if not a
            # known endpoint.
            self.send_response(404)
            self.end_headers()
            self.wfile.write(b"Not found")


if __name__ == "__main__":
    application = ApplicationBuilder().token(TOKEN).job_queue(queue).build()

    myid_handler = CommandHandler("myid", myid)
    application.add_handler(myid_handler)

    # 2196 is an unassigned port, but any port could work (as long as
    # it is not a well-known one, i.e., higher than 1023).
    httpd = HTTPServer(("localhost", 2196), MotionHandler)
    # Start the HTTP server is in a secondary thread, and use the main
    # thread for the bot.
    t = Thread(target=httpd.serve_forever)
    t.start()

    application.run_polling()

    # I guess we would not arrive here normally, but in case we do,
    # e.g., when testing interactively, do a proper cleanup.
    httpd.shutdown()
    t.join()
